#include "nxm_head.h"
#include "pcap.h"
#include "nxm_config_v0.6b.h"
#include "nxm_defines_v0.6b.h"
#include "nxm_globals_v0.6b.h"


extern int com_quit(); 
extern void process_input();
extern void update_windows(),notify_curses();
extern void set_ct_time_label(char *),set_ct_date_label(char *);
extern void write_sc(char *,int,int),write_cs(char *,int);
extern void delete_hash(),add_hash(char *,int);
extern void circle_copy(char *,int,char *,short *,short *,int);
void main_loop();
void init();
void main_loop();
pcap_handler packethandler(unsigned char *,const struct packetheader *,const unsigned char *);
unsigned short in_cksum(unsigned short *,int);
int unwrap_packet (unsigned char *,struct unwrap *);
void read_logstat(char *);
void read_config(char *);
void read_service(char *);
char *decide_service0(int);

main(argc,argv)
int argc;
char *argv[];
{
	void (*window_update_function)();
	int disp_usage=0,c;

	if(getenv("DISPLAY")!=NULL) xv=1;
	while((c=getopt(argc,argv,"?hvcxC:")) != -1)
	{
		switch(c)
		{
			case 'h':
			case 'v':
			case '?':
				disp_usage=1;
				break;
			case 'c':
				xv=0;
				break;
			case 'x':
				xv=2;
				break;
			case 'C':
				cfile=optarg;
				break;
			default:
				break;
		}
	}
	if(disp_usage)
	{
		/* Print Usage Here */
		printf("NetXmon Version 0.6 Demo\n\tusage:\tnxm [options]\n\t\t[-c(curses interface)]\n\t\t[-x(xterm interface)]\n\t\t[-C config_file(default is ./nxm.conf)]\n\tdefault will use xview interface.\n");
		com_quit();
	}

	if(geteuid()!=0)
	{
		fprintf(stderr,"Note: Should running with root privileges.\n");
		com_quit();
	}
	init();
	if(xv)
	{
		create_windows(argc,argv);
		window_update_function=update_windows;
	}else
	{
		init_screen();
		window_update_function=notify_curses;
	}
	for(;;)
	{
		(*window_update_function)();
		main_loop();
	}
}

void init()
{
	int i,semval;
	bpf_u_int32 localnet, netmask;
	int SUPPORTED=0;
	struct bpf_program fcode;
	char *cmdbuf,ebuf[128];
	int Oflag=1;
	union semun semctl_arg;
	char *dev;

	umask(027);
	signal(SIGHUP,SIG_IGN);
	signal(SIGINT,com_quit);
	signal(SIGSEGV,com_quit);
	time(&gtime);

	read_config(cfile);
	read_service(servfile);
	read_logstat(logstatfile);
	if(connname != NULL)
	{
		if((logcon = fopen(connname, "a+")) < 0)
		{
			perror("fopen");
			fprintf(stderr,"Error in open %s\n",connname);
		}
	}

#ifdef ICMP_PROCESS
	if(icmplog != NULL)
	{
		if((logicmp = fopen(icmplog, "a+")) < 0)
		{
			perror("fopen");
			fprintf(stderr,"Error in open %s\n",icmplog);
		}
	}
#endif
	

	for(i=0;i<HASHSIZE;i++)
		con_hash[i]=NULL;

#ifdef DEBUG
	if((debug_dev=fopen(DEBUG_DEVICE,"a"))<0)
	{
		printf("Couldn't open DEBUG device!\n");
		com_quit();
	}else
	{
		fprintf(debug_dev,"Debug begin for NetXMon\n");
	}
#endif

	if( xv == 2)
	{
		/* Add some share memory here */
		for(i=0; (shmid = shmget(SHMKEY+i,SHMSIZE+sizeof(pid_t),
			PERMS | IPC_CREAT | IPC_EXCL)) <0 && i < TRY_MAX; i++);
		if(i == TRY_MAX)
		{
			perror("shmget:");
			com_quit();
		}
		keyadd=i;
		if((swapspace=(char *)shmat(shmid,(char *)0,0)) == (char *)-1)
		{
			perror("shmat:");
			com_quit();
		}
		pid_term = (pid_t *) (swapspace+SHMSIZE);
		if((sem=semget(SEKEY1+i,1,PERMS | IPC_CREAT | IPC_EXCL)) < 0)
		{
			perror("semget");
			com_quit();
		}
		semctl_arg.val = 1;
		if (semctl(sem, 0, SETVAL, semctl_arg) < 0)
		{
			perror("semctl");
			com_quit();
		}
	
#ifdef DEBUG
		if ((semval = semctl(sem, 0, GETVAL, 0)) < 0)
		{
			perror("semctl");
			com_quit();
		}
		printf("semval is inited %d\n",semval);
#endif
		FD_ZERO(&allset);
	}

	if((dev=pcap_lookupdev(NULL))==NULL)
	{
		printf("No network devices found.Exit.\n");
		com_quit();
	}

#ifdef DEBUG
	printf("Network devices is %s\n",dev);
#endif

	if(pcap_lookupnet(dev, &localnet, &netmask, ebuf) < 0)
	{
		printf("%s\n",ebuf);
		com_quit();
	}
	
	for(i=0;i<ETH_DEV_NR;i++)
	{
		if(strstr(dev,ETH_DEV[i]))
		{
			PROTO_HEAD=ETHERHEAD;
			printf("Supported ethernet device found. (%s)\n",dev);
			SUPPORTED=1;
			break;
		}
	}
	for(i=0;i<PPP_DEV_NR;i++)
	{
		if(strstr(dev,PPP_DEV[i]))
		{
			PROTO_HEAD=PPPHEAD;
			printf("Supported PPP device found. (%s)\n",dev);
			SUPPORTED=1;
			break;
		}
	}
	if(SUPPORTED==0)
	{
		printf("Network device unknown. Please read the document or
			contact with me. \n");
		com_quit();
	}

	if((dev_desc=pcap_open_live(dev,SNAPLEN,1,MSDELAY,NULL))==NULL)
	{
		printf("Couldn't open device.\n");
		com_quit();
	}

	if((cmdbuf=read_filter(filfile)) != NULL)
	{
		if (pcap_compile(dev_desc, &fcode, cmdbuf, Oflag, netmask) < 0)
		{
			printf("%s\n",pcap_geterr(dev_desc));
			com_quit();
		}
		if (pcap_setfilter(dev_desc, &fcode) < 0)
		{
			printf("%s\n",pcap_geterr(dev_desc));
			com_quit();
		}
	}
}


void main_loop()
{
	char buf[80];
	time_t timenow;

	/* This sentence is used to make the NEW CON label eased quickly,*/
	erase_issyn++;
	if(gtime!=lasttimeCT && mapped && current)
	{
		/* if current != NULL, xv =1 or xv=0 */
		if(xv==1)
		{
			strftime(buf, 80, "CT: %H:%M:%S", localtime(&gtime));
			set_ct_time_label(buf);
			strftime(buf, 80, "    %D", localtime(&gtime));
			set_ct_date_label(buf);
		}
		lasttimeCT=gtime;
	}
	if(current_input)
	{
		if(pcap_dispatch(dev_desc,PACKETS,packethandler,NULL)<0)
		{
			char *errorbuf;
			errorbuf=pcap_geterr(dev_desc);
			if(strstr(errorbuf,"Interrupted system call") == NULL)
			{
                        	printf("Capturing Packets Failed\n");
				fprintf(stderr, "pcap_dispatch: %s\n",pcap_geterr(dev_desc));
				com_quit();
			}
		}
	}else
	{
		/* Read packet from file Reference main.c #162 */
		/* FIRST, READ THE PACKET FROM THE LOG */
		char *sp_log=read_log(watchfd, &ptime);

		/* IF REACHED EOF, DELETE CURRENT */
		if (!sp_log && reachedEOF && current)
			delete_hash();

		/* NEXT, UPDATE OUR IDEA OF TIME */
		if (delta_time==0)
		{
			/*
			 * Update the packet time. This is confusing...
			 * Essentially, we update with "realtime", except we also
			 * need to account for skews when the user is 'pausing'
			 * or fast forwarding. In those cases, time moves faster
			 * than one tick per second
			 */
			if (ptime==0)
				(void)time(&gtime);
			else
			{
				gtime=ptime;
				delta_time=time(0)-gtime;
			}
		}else
		{
			time(&timenow);
			if (!paused)
			{
				if (gtime+delta_time<timenow)
				gtime++;
				if ((playspeed!=1)&&(!(gtime%playspeed)))
					delta_time-=(playspeed-1);
			}else
			{
				if(gtime+delta_time<timenow)
					delta_time++;
			}
		}
		if(sp_log != NULL)process_packet(sp_log);
		else if(xv == 2)
		{
			process_input();
		}
	}
}

pcap_handler packethandler(unsigned char *dummy,
			const struct packetheader *p_header,
			const unsigned char *sp)
{
	if(sp != NULL)process_packet(sp);
}

void process_packet(char *sp)
{
	char *buf[80];
	int proto;

	proto=unwrap_packet(sp,&info);

	if(proto==NO_IP)	return;
	if(proto==NO_IP_4)	return;

	IP_nr_of_packets++;
	if(proto==ICMP)
	{
		/* Here process ICMP Message */
#ifdef ICMP_PROCESS
		struct in_addr from, to;
		char logstr[256];
		if(logicmp != NULL)
		{
			from.s_addr=htonl(IPhead.source);
			time(&gtime);
			strftime(logstr,256,"%y%m%d%H%M%S,From ",localtime(&gtime));
			strncat(logstr,inet_ntoa(from),256);
			to.s_addr=htonl(IPhead.destination);
			fprintf(logicmp,"%s to %s:",logstr,inet_ntoa(to));
			switch(ICMPhead.type)
			{
				case 0: fprintf(logicmp,"%s\n",ICMP_TYPE_0);
					break;
				case 3: fprintf(logicmp,"%s\n",ICMP_TYPE_3);
					fprintf(logicmp,"\tError: %s\n",
					ICMP_type_3_code[ICMPhead.code]);
					/*print_iphead(&iphead_icmp,1);*/
					break;
				case 4: fprintf(logicmp,"%s\n",ICMP_TYPE_4);
					/*print_iphead(&iphead_icmp,1);*/
					break;
				case 5: fprintf(logicmp,"%s\n",ICMP_TYPE_5);
					fprintf(logicmp,"\tError: %s\n",
					ICMP_type_5_code[ICMPhead.code]);
					/*print_iphead(&iphead_icmp,1);*/
					break;
					case 8: fprintf(logicmp,"%s\n",ICMP_TYPE_8);
					break;
				case 11:fprintf(logicmp,"%s\n",ICMP_TYPE_11);
					fprintf(logicmp,"\tError: %s\n",
					ICMP_type_11_code[ICMPhead.code]);
					/*print_iphead(&iphead_icmp,1);*/
					break;
				case 12:fprintf(logicmp,"%s\n",ICMP_TYPE_12);
					/*print_iphead(&iphead_icmp,1);*/
					break;
				case 13:fprintf(logicmp,"%s\n",ICMP_TYPE_13);
					break;
				case 14: fprintf(logicmp,"%s\n",ICMP_TYPE_14);
					break;
				case 15: fprintf(logicmp,"%s\n",ICMP_TYPE_15);
					break;
				case 16: fprintf(logicmp,"%s\n",ICMP_TYPE_16);
					break;
				case 17: fprintf(logicmp,"%s\n",ICMP_TYPE_17);
					break;
				case 18: fprintf(logicmp,"%s\n",ICMP_TYPE_18);
					break;
				default: fprintf(logicmp,"Unknown ICMP type!\n");break;
			}
		}
#endif
		return;
	}

	if(proto==UDP)
	{
		UDP_nr_of_packets++;
		UDP_bytes_in_packets+=(info.UDP_len+info.DATA_len+info.IP_len);
		/* Here process UDP message */
		return;
	}
	if(proto!=TCP)return;
	TCP_nr_of_packets++;
	TCP_bytes_in_packets+=(info.TCP_len+info.DATA_len+info.IP_len);

	/* Here process TCP message */
	if((TCPhead.offset_flag & FIN) != 0) /* TCP FIN */
	{
		delete_hash();
	}else if((TCPhead.offset_flag & RST) != 0) /* TCP RST */
	{
		delete_hash();
	} else if(((TCPhead.offset_flag & SYN) !=0) &&
			((TCPhead.offset_flag & ACK) !=0))
		/* TCP SYN The Second SYN , We discard the first ACK */
	{
		add_hash(sp,1);
	}else if(info.DATA_len == 0) /* Don't process zero lenth packet */
	{
		/* At present simple return */
		return;
	}else /* Normal Process */
	{
		/* If current not equal NULL, then xv must be 0 or 1 */
		if(current && current->ips==IPhead.source
			&& current->ipc==IPhead.destination
			&& current->ports==TCPhead.source
			&& current->portc==TCPhead.destination)
		/* This is from server to client */
		{
			char *data=sp+PROTO_HEAD+info.IP_len+info.TCP_len;
			/*circle_copy(data, info.DATA_len,current->scbuf,
				&current->scbegin,&current->scend, 2048);*/
			write_sc(data,info.DATA_len,1);
			if(gtime!=lasttimeLP)
			{
				if(xv)
				{
					strftime(buf, 80, "LP: %H:%M:%S",
						localtime(&gtime));
					set_lt_time_label(buf);
					strftime(buf, 80, "    %D",
						localtime(&gtime));
					set_lt_date_label(buf);
				}
				lasttimeLP=gtime;
				current->last_packet = gtime;
			}
			switch(current->curlog)
			{
				case 1:
				{
					/* RAW log */
					struct log_packet lp;
					lp.time=gtime;
					lp.data_len=info.TOTAL_len;
					write(current->logfd,(char *)&lp,
						sizeof(struct log_packet));
					if (lp.data_len>0)
						write(current->logfd,sp,
							lp.data_len);
					break;
				}
				case 2:
				{
					if(info.DATA_len > 0)
						write(current->logfd,data,
							info.DATA_len);	
					break;
				}
				default:
					break;
			}
					
		}else if(current && current->ipc==IPhead.source
			&& current->ips==IPhead.destination
			&& current->portc==TCPhead.source
			&& current->ports==TCPhead.destination)
			/* From client to server */
		{
			char *data=sp+PROTO_HEAD+info.IP_len+info.TCP_len;
			/*circle_copy(data, info.DATA_len,current->csbuf,
				&current->csbegin,&current->csend, 80);*/
			write_cs(data,info.DATA_len);
			if(gtime!=lasttimeLP)
			{
				if(xv)
				{
					strftime(buf, 80, "LP: %H:%M:%S",
						localtime(&gtime));
					set_lt_time_label(buf);
					strftime(buf, 80, "    %D",
						localtime(&gtime));
					set_lt_date_label(buf);
				}
				lasttimeLP=gtime;
				current->last_packet = gtime;
			}
			switch(current->curlog)
			{
				case 1:
				{
					/* RAW log */
					struct log_packet lp;
					lp.time=gtime;
					lp.data_len=info.TOTAL_len;
					write(current->logfd,(char *)&lp,
						sizeof(struct log_packet));
					if (lp.data_len>0)
						write(current->logfd,sp,
							lp.data_len);
					break;
				}
				case 2:
				{
					if(info.DATA_len > 0)
						write(current->logfd,data,
							info.DATA_len);	
					break;
				}
				default:
					break;
			}
		}else
		{
			add_hash(sp,0);
		}
	}
 /* Here should add hijack source see main.c 283# */
}



unsigned short in_cksum(unsigned short *addr,int len)
{
	register int nleft = len;
	register unsigned short *w = addr; 
	register int sum = 0;
	unsigned short answer = 0;     

	while (nleft > 1)
	{
		sum += *w++;
		nleft -= 2;
	}
	if (nleft == 1)
	{
		*(unsigned char *)(&answer) = *(unsigned char *)w ;
		sum += answer;
	}
	sum = (sum >> 16) + (sum & 0xffff);
	sum += (sum >> 16);
	answer = ~sum;
	return(answer);
}

/* Since it is too much trouble to call ntohs,ntohl for so many times,
   I deicide to put them here at all */
int unwrap_packet (unsigned char *sp, struct unwrap *unwrapped) 
{ 
	int i;
	memcpy(&IPhead,(sp+PROTO_HEAD),sizeof(struct IP_header));

 	unwrapped->IP_len = (IPhead.verlen & 0xF) << 2;
	unwrapped->TCP_len = 0;
	unwrapped->UDP_len = 0;
	unwrapped->DATA_len = 0;
        
	if(NO_CHKSUM == 0)
	{
		sp[PROTO_HEAD+10]=0;       /* reset checksum to zero, Q&D way*/
		sp[PROTO_HEAD+11]=0;
		if(in_cksum((sp+PROTO_HEAD),unwrapped->IP_len)!=IPhead.checksum)
		{
#ifdef DEBUG
			printf("Packet dropped... (invalid IP chksum)\n");
			printf("%X   %X (len %d)\n",in_cksum((sp+PROTO_HEAD),unwrapped->IP_len),IPhead.checksum,unwrapped->IP_len);
#endif

			return; /* is NO_IP previously */
		}
		if(0)
		{
#ifdef DEBUG
			printf("Packet dropped... (invalid IP version)\n");
#endif

			return; /* return NO_IP_4;*/
		}
		memcpy((sp+PROTO_HEAD),&IPhead,sizeof(struct IP_header));
	}

	/* I only change everything I needed */
	IPhead.source=ntohl(IPhead.source);
	IPhead.destination=ntohl(IPhead.destination);

	if(IPhead.protocol == TCP )		             /* TCP */
	{
		memcpy(&TCPhead,(sp+PROTO_HEAD+(unwrapped->IP_len)),
						sizeof(struct TCP_header));

		/* A little joking! I won't change everything but only I
			take interest in it */
		TCPhead.source=ntohs(TCPhead.source);
		TCPhead.destination=ntohs(TCPhead.destination);
		TCPhead.offset_flag=ntohs(TCPhead.offset_flag);

		unwrapped->TCP_len = TCPhead.offset_flag & 0xF000;
		unwrapped->TCP_len >>= 10; 
		unwrapped->DATA_len = ntohs(IPhead.length) -
				(unwrapped->IP_len) - (unwrapped->TCP_len); 
		unwrapped->TOTAL_len = PROTO_HEAD+unwrapped->IP_len+
				unwrapped->TCP_len+unwrapped->DATA_len;
		return TCP;
	}
	if(IPhead.protocol == ICMP )		             /* ICMP */
	{
		memcpy(&ICMPhead,(sp+PROTO_HEAD+(unwrapped->IP_len)),
						sizeof(struct ICMP_header));
		unwrapped->ICMP_len = ICMP_HEADLENGTH;
		unwrapped->DATA_len = ntohs(IPhead.length) -
				(unwrapped->IP_len) - (unwrapped->ICMP_len); 
		unwrapped->TOTAL_len = PROTO_HEAD+unwrapped->IP_len+
				unwrapped->ICMP_len+unwrapped->DATA_len;
		return ICMP; 
	}
	if(IPhead.protocol == UDP )		               /* UDP */
	{
		memcpy(&UDPhead,(sp+PROTO_HEAD+(unwrapped->IP_len)),
						sizeof(struct UDP_header));
		
		/* A little joking! I won't change everything but only I
			take interest in it */
		UDPhead.source=ntohs(UDPhead.source);
		UDPhead.destination=ntohs(UDPhead.destination);

		unwrapped->UDP_len = UDP_HEADLENGTH;
		unwrapped->DATA_len = ntohs(IPhead.length) -
				(unwrapped->IP_len) - (unwrapped->UDP_len); 
		unwrapped->TOTAL_len = PROTO_HEAD+unwrapped->IP_len+
				unwrapped->UDP_len+unwrapped->DATA_len;
		return UDP; 
	}
	return -1; 
}

char *read_log(fd,ptime)
int fd;
time_t *ptime; /* packet time */

{
	struct log_packet newpacket;
	int lenread;
	int i=1;
	char *pkt_databuf;
	static int packetsent;
	static time_t thepack_time;
	static char tbuf[4096];

	/* we want to read the first packet to get our first time stamp, but we
	 * also want to read if the packet we have has been sent
	 */
 	if (reachedEOF)
	{
		/* Hack to keep it from busy waiting on an EOF */
		musleep(100000);
		return(NULL);
	}
	if ((!pkt_time)||(packetsent))
	{
		/* first time we'll set the read to nonblocking */
		if (!pkt_time) fcntl(fd,F_SETFL,O_NONBLOCK);
		lenread = read(fd, (char *)&newpacket,
			sizeof(struct log_packet));
		if (lenread == sizeof(struct log_packet))
		{
			if (newpacket.data_len > 0)
			{
				pkt_databuf = tbuf;
				if ((read(fd, pkt_databuf, newpacket.data_len))
					!= newpacket.data_len)
				{
					switch (errno)
					{
						case 0:
							reachedEOF = 1;
							playspeed = 1;
							return(NULL);
						case EINTR:
							return(NULL);
						default:
							perror("read packet data log:");
							com_quit();
					}
				}
			}
			thepack_time=newpacket.time;
			packetsent = 0;
			if (!pkt_time) pkt_time =thepack_time;
		}else if (errno == EWOULDBLOCK) 
			return(NULL);
		else
		{
			switch(errno)
			{
				case 0:
					reachedEOF = 1;
					playspeed = 1;
					return(NULL);
				case EINTR:
					return(NULL);
				default:
					perror("read packet from log file:");
					com_quit();
			}
	  	}
  	}
	*ptime=pkt_time;
	if ((gtime >= thepack_time)&&(!packetsent))
	{
		packetsent = 1;
		*ptime=thepack_time;
		return tbuf;
	}else
	{
		musleep(100000);
		/* Sleep 1/10th second--stop busy wait, simulate network */
		return NULL;
	}
}

/* Stupid Solaris... This is their official hack to do subsecond sleeps... 
 *
 * We're using this rather than the real usleep under 4.x because it 
 * interferes with setitimer usage
 */

musleep(useconds)
unsigned long int useconds;
{
struct timeval delay;

delay.tv_sec = 0;
delay.tv_usec = useconds;

if (select(0, NULL, NULL, NULL, &delay)==-1 && errno!=EINTR)
	perror("select");
return(0);
}

void read_config(char * fname)
{
	FILE *fin;
	int i=0;
	char buf[100];

	if((fin=fopen(fname,"r"))==NULL)
	{
		perror("fopen:");
		return;
	}
	while(fgets(buf,100,fin) != NULL)
	{
		while(strlen(buf) == 99 && buf[98] != '\n' && fgets(buf,100,fin) != NULL);
		if((buf[0]!='#') && buf[0]!='\n' && strcmp(buf,""))
		{
			i++;
			buf[strlen(buf)-1]=0;
			if(i == 1)
			{
				servfile=strdup(buf);
			}else if(i == 2)
			{
				filfile=strdup(buf);
			}else if(i == 3)
			{
				logstatfile=strdup(buf);
			}else if(i == 4)
			{
				connname=strdup(buf);
			}else if(i == 5)
			{
				TERM=strdup(buf);
			}
#ifdef ICMP_PROCESS
			else if(i == 6)
			{
				icmplog=strdup(buf);
			}
#endif
			else
			{
				fclose(fin);
				return;
			}
		}
	}
	fclose(fin);
}


void read_service(char * fname)
{
	FILE *fin;
	int i;
	char buf[1024];
	struct Service *p;
	char *str1,*str2,*str3;

	for(i=0;i<1024;i++)
	{
		serv[i]=NULL;
	}
	if((fin=fopen(fname,"r"))==NULL)
	{
		perror("fopen:");
		return;
	}
	while(fgets(buf,1024,fin) != NULL)
	{
		while(strlen(buf) == 1023 && buf[1022] != '\n' && fgets(buf,1024,fin) != NULL);
		if(buf[0] != '#' && buf[0] != '\n' && strcmp(buf,""))
		{
			str1=strtok(buf,"\t \n\/");
			str2=strtok(NULL,"\t \n\/");
			str3=strtok(NULL,"\t \n\/");
			if(str1 == NULL || str2 == NULL || str3 == NULL)
				continue;
			if(!strcmp(str3,"tcp"))
			{
				i=atoi(str2);
				if(i<1024) serv[i]=strdup(str1);
				else 
				{
					if(other_serv==NULL)
					{
						other_serv=(struct Service *)
						malloc(sizeof(struct Service));
						if(other_serv==NULL)
						{
							perror("malloc:");
							com_quit();
						}
						other_serv->pnum=i;
						other_serv->ser=strdup(str1);
						other_serv->next=NULL;
					}else
					{
						p=other_serv;
						while(p->next!=NULL)
						{
							p=p->next;
						}
						p->next=(struct Service *)
						malloc(sizeof(struct Service));
						p=p->next;
						if(p==NULL)
						{
							perror("malloc:");
							com_quit();
						}
						p->pnum=i;
						p->ser=strdup(str1);
						p->next=NULL;
					}
				}
			}else
			{
			}
		}
	}
	fclose(fin);
}

int decide_service(int port1,int port2,char *pserv,int n)
{
	if(port1<port2)
	{
		strncpy(pserv,decide_service0(port1),n);
		if(!strcmp(pserv,"UNKNOWN"))
		{
			strncpy(pserv,decide_service0(port2),n);
			return 1;
		}else return 0;
	}else if(port2<port1)
	{
		strncpy(pserv,decide_service0(port2),n);
		if(!strcmp(pserv,"UNKNOWN"))
		{
			strncpy(pserv,decide_service0(port1),n);
			return 0;
		}else return 1;
	}
}


char *decide_service0(int port)
{
	struct Service *p;
	if(port<1024)
	{
		if(serv[port] != NULL)
		return serv[port];
		else
		return "UNKNOWN";
	}
	else if(other_serv==NULL)
		return "UNKNOWN";
	else
	{
		p=other_serv;
		while(p!=NULL)
		{
			if(p->pnum==port)
				return p->ser;
			p=p->next;
		}
		return "UNKNOWN";
	}
}

char * read_filter(fname)
char *fname;
{
	register int fd, cc;
	register char *cp;
	struct stat buf;

	if ((fd=open(fname, O_RDONLY)) < 0)
	{
		perror("open");
		return NULL;
	}

	if (fstat(fd, &buf) < 0)
	{
		perror("open");
		return NULL;
	}

	cp = malloc((u_int)buf.st_size + 1);
	if((cc = read(fd, cp, (int)buf.st_size)) < 0)
	{
		perror("read");
		return NULL;
	}
	if (cc != buf.st_size)
	{
		perror("read");
		return NULL;
	}
	cp[(int)buf.st_size] = '\0';
	return (cp);
}

void read_logstat(fname)
char *fname;
{
	FILE *fin;
	int i;
	char buf[100];
	struct Log_init *p=NULL;
	char *str1,*str2,*str3,*str4,*str5,*pch;

	if((fin=fopen(fname,"r"))==NULL)
	{
		perror("fopen:");
		return;
	}
	while(fgets(buf,100,fin) != NULL)
	{
		while(strlen(buf) == 99 && buf[98] != '\n' && fgets(buf,100,fin) != NULL);
		if((buf[0] != '#') && buf[0] != '\n' && strcmp(buf,""))
		{
			str1=strtok(buf,"\t \n");
			str2=strtok(NULL,"\t \n");
			str3=strtok(NULL,"\t \n");
			str4=strtok(NULL,"\t \n");
			str5=strtok(NULL,"\t \n");
				
			if(str1 == NULL || str2 == NULL ||
				str3 == NULL || str4 == NULL || str5 == NULL)
				continue;

#ifdef DEBUG
			printf("%s\t%s\t%s\n",str1,str2,str3,str4);
#endif

			if(loginit == NULL && 
				(p=loginit=(struct Log_int *)
				malloc(sizeof(struct Log_init))) == NULL)
			{
				perror("malloc");
				return;
			}else if(loginit != NULL)
			{
				if((p->next= (struct Log_init *)
					malloc(sizeof(struct Log_init))) == NULL)
				{
					perror("malloc");
					return;
				}
				p=p->next;
			}

			p->ips_b=ntohl(inet_addr(strtok(str1,"-")));
			if((pch=strtok(NULL,"-")) != NULL)
			{
				p->ips_e=ntohl(inet_addr(pch));
			}else
			{
				p->ips_e=p->ips_b;
			}
			if((pch=strchr(str2,'-')) != NULL)
			{
				sscanf(str2,"%u-%u",&(p->ports_b),&(p->ports_e));
			}else
			{
				sscanf(str2,"%u",&(p->ports_b));
				p->ports_e=p->ports_b;
			}
			
			p->ipd_b=ntohl(inet_addr(strtok(str3,"-\n")));
			if((pch=strtok(NULL,"-")) != NULL)
			{
				p->ipd_e=ntohl(inet_addr(pch));
			}else
			{
				p->ipd_e=p->ipd_b;
			}
			if((pch=strchr(str4,'-')) != NULL)
			{
				sscanf(str4,"%u-%u",&(p->portd_b),&(p->portd_e));
			}else
			{
				sscanf(str4,"%u",&(p->portd_b));
				p->portd_e=p->portd_b;
			}
			p->logstat=-1;
			if(!strcmp(str5,"NL_S")) p->logstat=-1;
			if(!strcmp(str5,"LM_S")) p->logstat=0;
			if(!strcmp(str5,"LF_S")) p->logstat=1;
			if(!strcmp(str5,"LFT_S")) p->logstat=2;
			if(!strcmp(str5,"LFM_S")) p->logstat=3;
			if(!strcmp(str5,"LFTM_S")) p->logstat=4;
		}
	}
	fclose(fin);
}

int decide_logstat(ip1,ip2,port1,port2)
unsigned long int ip1,ip2;
unsigned int port1,port2;
{
	struct Log_init *p;
	p=loginit;
	while(p != NULL)
	{
		if(((p->ips_b <= ip1 && p->ips_e >= ip1)
			&& (p->ipd_b <= ip2 && p->ipd_e >= ip2)
			&& (p->ports_b <= port1 && p->ports_e >= port1)
			&& (p->portd_b <= port2 && p->portd_e >= port2))
		||
			((p->ipd_b <= ip1 && p->ipd_e >= ip1)
			&& (p->ips_b <= ip2 && p->ips_e >= ip2)
			&& (p->portd_b <= port1 && p->portd_e >= port1)
			&& (p->ports_b <= port2 && p->ports_e >= port2)))
		{
			break;
		}else
		{
			p=p->next;
		}
	}
	if( p != NULL )
	{
		if( xv == 2 )
			return p->logstat;
		else
		{
			if( p->logstat == 0 ) return -2;
			if( p->logstat == 1 ) return  1;
			if( p->logstat == 2 ) return  2;
			if( p->logstat == 3 ) return  1;
			if( p->logstat == 4 ) return  2;
		}
	}
	if( xv == 2 )
		return -1;
	else return -2;
}
